/* eslint-disable */
/**
 * no-mixed-requires one-var
 * Perform some low level check to determine the installer
 */
const Generator = require('./base.js'),
  _ = require('lodash'),
  chalk = require('chalk'),
  exec = require('child_process').exec,
  when = require('when');
// Other import
const env = process.env;
// Config = require(join(__dirname, 'config.json'));

module.exports = class extends Generator {
  constructor(args, opts) {
    super(args, opts);
    // Run the check yarn here?
    this._checkIfYarnInstalled().then(result => {
      this.baseConfig.yarnInstalled = result;
      this.props = _.extend(this.props, { yarnInstalled: result });
    });
    // Run the check to see if cnpm installed
    this._checkIfCnpmInstalled().then(result => {
      this.baseConfig.cnpmInstalled = result;
      this.props = _.extend(this.props, { cnpmInstalled: result });
    });
  }
  /**
   * check if this generator actually installed
   */
  _checkIfAlreadyInstalled() {
    // just checking if lang is presented
    return this.config.get('lang');
  }
  /**
   * actually running the installer
   * @param {mixed} args (packages, opts, callback)
   * @return {undefined} nothing
   */
  _runInstaller(...args) {
    if (env.NODE_ENV === 'test') {
      this.npmInstall();
    } else {
      this.log(chalk.yellow(this.t('runningInstaller', { installer: this.installer })));
      this._getInstaller(...args);
    }
  }
  /**
   * Creating the installer
   * @param {mixed} arguments to pass packages, opt, callback
   */
  _getInstaller(...args) {
    const i = this.props.installer;
    return i === 'cnpm'
      ? this.runInstaller('cnpm', ...args)
      : this[this.props.installer](...args);
  }
  /**
   * If yarn install or they are using cn in the lang option
   */
  _askForInstaller() {
    const choices = this._getInstallerChoices();
    if (choices.length === 1) {
      return Promise.resolve();
    }
    return this.prompt({
      type: 'list',
      choices: choices,
      name: 'installer',
      message: 'whichInstallerToUse',
      default: 'npmInstall'
    }).then(prompt => {
      this.props.installer = prompt.installer;
    });
  }
  /**
   * Get available installers
   * @return {array} installers
   */
  _getInstallerChoices() {
    const installChoices = [
      {
        name: 'npm',
        value: 'npmInstall'
      }
    ];
    if (this.props.yarnInstalled) {
      installChoices.push({
        name: 'yarn',
        value: 'yarnInstall'
      });
    }
    if (this.lang === 'cn' && this.props.cnpmInstalled) {
      installChoices.push({
        name: 'cnpm',
        value: 'cnpm'
      });
    }
    return installChoices;
  }
  /**
   * @param {string} cmd command
   * @return {object} promise or resolved
   */
  _runCommand(cmd) {
    return when.promise((resolver, rejecter) => {
      exec(cmd, (error, stdout, stderr) => {
        if (error) {
          return rejecter(`${error}`);
        }
        return resolver({
          stdout: `${stdout}`,
          stderr: `${stderr}`
        });
      });
    });
  }
  /**
   * Check if certain thing installed, use to check yarn and cnpm
   * but don't do this when it's in a test environment
   * @param {string} cmd command to run
   * @return {object} promise result
   */
  _checkIfCmdExist(cmd) {
    return when.promise(resolver => {
      if (env.NODE_ENV === 'test') {
        return resolver(false);
      }
      return this._runCommand(cmd)
        .then(() => {
          // The same bug surface the property set earlier
          // disappear the prompt so setting this.props.installer is useless
          resolver(true);
        })
        .catch(() => {
          resolver(false);
        });
    });
  }

  /**
   * Check if yarn installed or not
   * @return {object} promise
   */
  _checkIfYarnInstalled() {
    return this._checkIfCmdExist('yarn versions');
  }
  /**
   * Check if cnpm installed or not
   * @return {object} promise
   */
  _checkIfCnpmInstalled() {
    return this._checkIfCmdExist('cnpm -v');
  }
};
